package mongo

import (
	"context"
	siteGroupMarginConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/SiteGroupMarginConfig"
	brandDataModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/brand_data"
	productModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/product"
	siteComboMarginConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/site_combo_margin_config"
	siteFloatMarginConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/site_float_margin_config"
	sitePlatformPriceConfigModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/site_platform_price_config"
	subjectModel "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/subject"
	reflect2 "git.chillcy.com/golang/colorway/pkg/reflect"
	"reflect"
)

const DatabaseColorWay = "color_way"
const DatabaseMpc = "mpc"

type Manager struct {
	ProductModel                 productModel.ProductModel
	SitePlatformPriceConfigModel sitePlatformPriceConfigModel.SitePlatformPriceConfigModel
	SubjectModel                 subjectModel.SubjectModel
	SiteComboMarginConfigModel   siteComboMarginConfigModel.SiteComboMarginConfigModel
	SiteFloatMarginConfigModel   siteFloatMarginConfigModel.SiteFloatMarginConfigModel
	BrandDataModel               brandDataModel.BrandDataModel
	SiteGroupMarginConfigModel   siteGroupMarginConfigModel.SiteGroupMarginConfigModel
}

func NewManager(mongoUrl string) *Manager {
	return &Manager{
		ProductModel:                 productModel.NewProductModel(mongoUrl, DatabaseColorWay, productModel.Collection),
		SitePlatformPriceConfigModel: sitePlatformPriceConfigModel.NewSitePlatformPriceConfigModel(mongoUrl, DatabaseMpc, sitePlatformPriceConfigModel.Collection),
		SubjectModel:                 subjectModel.NewSubjectModel(mongoUrl, DatabaseMpc, subjectModel.Collection),
		SiteComboMarginConfigModel:   siteComboMarginConfigModel.NewSiteComboMarginConfigModel(mongoUrl, DatabaseMpc, siteComboMarginConfigModel.Collection),
		SiteFloatMarginConfigModel:   siteFloatMarginConfigModel.NewSiteFloatMarginConfigModel(mongoUrl, DatabaseMpc, siteFloatMarginConfigModel.Collection),
		BrandDataModel:               brandDataModel.NewBrandDataModel(mongoUrl, DatabaseMpc, brandDataModel.Collection),
		SiteGroupMarginConfigModel:   siteGroupMarginConfigModel.NewSiteGroupMarginConfigModel(mongoUrl, DatabaseMpc, brandDataModel.Collection),
	}
}

func (receiver *Manager) StartCache() error {
	ctx := context.TODO()
	args := []reflect.Value{
		reflect.ValueOf(ctx),
	}
	return reflect2.MemberMethodCall(receiver, "StartCache", args)
}
