package model

import (
	"git.chillcy.com/golang/chillcy/pkg/gender"
	"git.chillcy.com/golang/chillcy/pkg/slice"
	"git.chillcy.com/golang/colorway/pkg/types"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

const Collection = "site_combo_margin_config"

type SiteComboMarginConfig struct {
	Id         primitive.ObjectID `bson:"_id,omitempty"`
	Seller     types.Seller       `bson:"seller"`
	SiteId     string             `bson:"siteId"`
	Brands     []string           `bson:"brands"`
	Genders    gender.Genders     `bson:"genders"`
	Categories []string           `bson:"categories"`
	Margin     float64            `bson:"margin"`
	Memos      types.Memos        `bson:"memos"`
	Enable     bool               `bson:"enable"` // 是否启用`
}

func (receiver SiteComboMarginConfig) IsGenderMatch(genders gender.Genders) bool {
	if len(receiver.Genders) <= 0 {
		return true
	}
	genders = slice.RemoveDuplicateItem(genders)
	for _, g := range receiver.Genders {
		if g == gender.GenderAdult {
			v := slice.ContainsElems(genders, []gender.Gender{
				gender.GenderMan,
				gender.GenderWoman,
			}) || onlyHave(genders, g)
			if v {
				return true
			}
		}
		if g == gender.GenderChild {
			v := slice.ContainsElems(genders, []gender.Gender{
				gender.GenderBoy,
				gender.GenderGirl,
			}) || onlyHave(genders, g)
			if v {
				return true
			}
		}
		if onlyHave(genders, g) {
			return true
		}
	}

	return false
}

func onlyHave(arr []gender.Gender, v gender.Gender) bool {
	if len(arr) == 1 && arr[0] == v {
		return true
	}
	return false
}
