package model

import (
	"encoding/json"
	"git.chillcy.com/golang/colorway/pkg/types"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

const Collection = "site_float_margin_config"

type SiteFloatMarginConfig struct {
	Id       *primitive.ObjectID `bson:"_id,omitempty"`
	Seller   types.Seller        `bson:"seller"`
	Season   string              `bson:"season"`
	Category string              `bson:"category"`
	Brand    string              `bson:"brand"`
	Enable   bool                `bson:"enable"` // 是否启用
	Memos    types.Memos         `bson:"memos"`
}

type ConditionRule struct {
	Type   string        `bson:"type"`
	Params []ConditionKv `bson:"params"`
}

func (cr ConditionRule) Get(k string) string {
	for _, p := range cr.Params {
		if p.Key == k {
			return p.Value
		}
	}

	return ""
}

const ConditionKeySeasonCO = "season_co"   // CO经典款
const ConditionKeySeasonNow = "season_now" // 当季款
const ConditionKeySeasonOld = "season_old" // 过季款

const ConditionKeyBrandTop = "brand_top"       // 头部品牌
const ConditionKeyBrandMiddle = "brand_middle" // 腰部品牌
const ConditionKeyBrandTail = "brand_tail"     // 尾部品牌

const ConditionKeyCategoryClothing = "category_clothing"     // 服装品类
const ConditionKeyCategoryShoes = "category_shoes"           // 鞋靴品类
const ConditionKeyCategoryBags = "category_bag"              // 包袋品类
const ConditionKeyCategoryAccessories = "category_accessory" // 配饰品类

type ConditionKv struct {
	Key   string `bson:"key"`
	Value string `bson:"value"`
}

func (c *SiteFloatMarginConfig) Explain() ([]ConditionRule, error) {
	rules := make([]ConditionRule, 0)
	if c.Season != "" {
		rule, err := convertToRule("season", c.Season)
		if err != nil {
			return nil, err
		}
		rules = append(rules, *rule)
	}
	if c.Category != "" {
		rule, err := convertToRule("category", c.Category)
		if err != nil {
			return nil, err
		}
		rules = append(rules, *rule)
	}
	if c.Brand != "" {
		rule, err := convertToRule("brand", c.Brand)
		if err != nil {
			return nil, err
		}
		rules = append(rules, *rule)
	}
	return rules, nil
}

func convertToRule(_type, content string) (*ConditionRule, error) {
	var jsonContent map[string]interface{}
	err := json.Unmarshal([]byte(content), &jsonContent)
	if err != nil {
		return nil, err
	}
	rule := ConditionRule{
		Type: _type,
	}
	for key, value := range jsonContent {
		rule.Params = append(rule.Params, ConditionKv{
			Key:   key,
			Value: value.(string),
		})
	}
	return &rule, nil
}
