package model

import (
	"git.chillcy.com/golang/colorway/internal/pkg/database/mongo/cache"
	"github.com/zeromicro/go-zero/core/stores/mon"
)

var _ SubjectModel = (*customSubjectModel)(nil)

type (
	// SubjectModel is an interface to be customized, add more methods here,
	// and implement the added methods in customSubjectModel.
	SubjectModel interface {
		subjectModel
		cache.Cache[Subject]
		cache.ChangeStream[Subject]
	}

	customSubjectModel struct {
		*defaultSubjectModel
		*cache.Base[Subject]
	}
)

// NewSubjectModel returns a model for the mongo.
func NewSubjectModel(url, db, collection string) SubjectModel {
	conn := mon.MustNewModel(url, db, collection)
	defaultModel := newDefaultSubjectModel(conn)
	customModel := &customSubjectModel{
		defaultSubjectModel: defaultModel,
	}
	customModel.Base = cache.NewBase[Subject](defaultModel, customModel)
	return customModel
}

func (receiver *customSubjectModel) DocumentEqual(a, b Subject) bool {
	return a.Id.Hex() == b.Id.Hex()
}
