package mysql

import (
	"fmt"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mysql/repository"
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"reflect"
)

type Manager struct {
	CompleteSpuRepository repository.CompleteSpuInterface
	ColorSpuRepository    repository.ColorSpuInterface
	SkuRepository         repository.SkuInterface
	CategoryRepository    repository.CategoryInterface
	CompositionRepository repository.CompositionInterface
	FitRepository         repository.FitInterface
	GenderRepository      repository.GenderInterface
	ImageRepository       repository.ImageInterface
	ModelRepository       repository.ModelInterface
	ProductRepository     repository.ProductInterface
}

func (receiver *Manager) init() error {
	return receiver.methodCall([]string{"AutoMigrate"})
}

func (receiver *Manager) methodCall(methodNames []string) error {
	v := reflect.ValueOf(receiver).Elem() // 获取指针指向的值
	t := reflect.TypeOf(receiver).Elem()  // 获取指针指向的类型
	for _, methodName := range methodNames {
		for i := 0; i < v.NumField(); i++ {
			field := v.Field(i)
			fieldType := t.Field(i)

			// 检查字段是否为 nil
			if field.IsNil() {
				return fmt.Errorf("字段 %s 为 nil，跳过\n", fieldType.Name)
			}

			// 检查是否有 methodName 方法
			method := field.MethodByName(methodName)
			if !method.IsValid() {
				return fmt.Errorf("字段 %s 没有 %s 方法\n", fieldType.Name, methodName)
			}

			// 调用 methodName 方法
			fmt.Printf("调用 %s.%s()\n", fieldType.Name, methodName)
			method.Call(nil)
		}
	}

	return nil

}

func NewManager(dsn string) (*Manager, error) {
	db, err := gorm.Open(mysql.Open(dsn), &gorm.Config{
		DisableForeignKeyConstraintWhenMigrating: false, // 启用外键约束创建
	})
	if err != nil {
		return nil, err
	}
	manager := &Manager{
		CompleteSpuRepository: repository.NewCompleteSpu(db),
		ColorSpuRepository:    repository.NewColorSpu(db),
		SkuRepository:         repository.NewSku(db),
		CategoryRepository:    repository.NewCategory(db),
		CompositionRepository: repository.NewComposition(db),
		FitRepository:         repository.NewFit(db),
		GenderRepository:      repository.NewGender(db),
		ImageRepository:       repository.NewImage(db),
		ModelRepository:       repository.NewModel(db),
		ProductRepository:     repository.NewProduct(db),
	}
	err = manager.init()
	return manager, err
}
