package model

import "gorm.io/gorm"

type CompleteSpu struct {
	gorm.Model
	StyleNumber string `gorm:"uniqueIndex; size:50; not null"`
	Title       Title  `gorm:"embedded;embeddedPrefix:title_"`
	Description string `gorm:"type:text"`
	Season      string `gorm:"size:20"`
	BrandId     string `gorm:"size:50"`
	Care        string `gorm:"type:text"`
	MadeIn      string `gorm:"size:50"`
	HsCode      string `gorm:"size:50"`
	InfoSelect  string `gorm:"not null; size:24"` // 商品信息默认选择哪个商品

	// 外键
	// 多对一
	ColorSpus    ColorSpus    `gorm:"foreignKey:CompleteSpuStyleNumber; references:StyleId; constraint:OnUpdate:CASCADE,OnDelete:CASCADE;"`
	Compositions Compositions `gorm:"foreignKey:CompleteSpuStyleNumber; references:StyleId; constraint:OnUpdate:CASCADE,OnDelete:CASCADE;"`
	Models       Models       `gorm:"foreignKey:CompleteSpuStyleNumber; references:StyleId; constraint:OnUpdate:CASCADE,OnDelete:CASCADE;"`
	Fits         Fits         `gorm:"foreignKey:CompleteSpuStyleNumber; references:StyleId; constraint:OnUpdate:CASCADE,OnDelete:CASCADE;"`
	// 多对多
	Genders    Genders    `gorm:"many2many:completespu_genders;foreignKey:StyleId; References:Code;"`
	Categories Categories `gorm:"many2many:completespu_categories;foreignKey:StyleId; References:Code;"`
}

type CompleteSpus []CompleteSpu

func (receiver CompleteSpus) GetColorSpus() ColorSpus {
	var colorSpus ColorSpus
	for _, spu := range receiver {
		colorSpus = append(colorSpus, spu.ColorSpus...)
	}
	return colorSpus
}

func (receiver CompleteSpus) GetCompositions() Compositions {
	var compositions Compositions
	for _, spu := range receiver {
		compositions = append(compositions, spu.Compositions...)
	}
	return compositions
}

func (receiver CompleteSpus) GetModels() Models {
	var models Models
	for _, spu := range receiver {
		models = append(models, spu.Models...)
	}
	return models
}

func (receiver CompleteSpus) GetFits() Fits {
	var fits Fits
	for _, spu := range receiver {
		fits = append(fits, spu.Fits...)
	}
	return fits
}

func (receiver CompleteSpus) GetStyleNumbers() []string {
	var styleNumbers []string
	for _, spu := range receiver {
		styleNumbers = append(styleNumbers, spu.StyleNumber)
	}
	return styleNumbers
}

func (receiver CompleteSpus) GetGenders() Genders {
	var genders Genders
	for _, spu := range receiver {
		genders = append(genders, spu.Genders...)
	}
	return genders
}

func (receiver CompleteSpus) GetCategories() Categories {
	var categories Categories
	for _, spu := range receiver {
		categories = append(categories, spu.Categories...)
	}
	return categories
}

type Title struct {
	En string `gorm:"size:100"`
	Cn string `gorm:"size:100"`
}
