package option

import "gorm.io/gorm"

// Query 查询选项
type Query func(*gorm.DB) *gorm.DB

// Where 条件查询
func Where(condition interface{}, args ...interface{}) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Where(condition, args...)
	}
}

// Order 排序
func Order(value interface{}) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Order(value)
	}
}

// Limit 限制数量
func Limit(limit int) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Limit(limit)
	}
}

// Offset 偏移量
func Offset(offset int) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Offset(offset)
	}
}

// Preload 预加载
func Preload(query string, args ...interface{}) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Preload(query, args...)
	}
}

// Joins 关联查询
func Joins(query string, args ...interface{}) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Joins(query, args...)
	}
}

// Select 选择字段
func Select(query interface{}, args ...interface{}) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Select(query, args...)
	}
}

// Group 分组
func Group(name string) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Group(name)
	}
}

// Having Having条件
func Having(query interface{}, args ...interface{}) Query {
	return func(db *gorm.DB) *gorm.DB {
		return db.Having(query, args...)
	}
}
