package repository

import (
	"git.chillcy.com/golang/colorway/internal/pkg/database/mysql/model"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
)

type CategoryInterface interface {
	BaseInterface
	UpsertManyByCode(categories model.Categories) error
}

type Category struct {
	*Base[model.Category]
}

func NewCategory(db *gorm.DB) *Category {
	return &Category{
		Base: NewBase[model.Category](db),
	}
}

func (receiver *Category) UpsertManyByCode(categories model.Categories) error {
	result := receiver.db.Clauses(clause.OnConflict{
		Columns: []clause.Column{{Name: "code"}},
		DoUpdates: clause.Assignments(map[string]interface{}{
			"name":        clause.Column{Name: "name"},
			"parent_code": clause.Column{Name: "parent_code"},
			"updated_at":  clause.Expr{SQL: "NOW()"}, // 手动更新时间戳
		}),
	}).Create(&categories)
	return result.Error
}
