package transaction

import (
	"context"
	"gorm.io/gorm"
)

// Manager 事务管理器
type Manager struct {
	db *gorm.DB
}

// NewManager 创建事务管理器
func NewManager(db *gorm.DB) *Manager {
	return &Manager{db: db}
}

// WithTransaction 在事务中执行操作
func (tm *Manager) WithTransaction(ctx context.Context, fn func(tx *gorm.DB) error) error {
	return tm.db.WithContext(ctx).Transaction(fn)
}

// WithTransactionResult 在事务中执行操作并返回结果
//func (tm *Manager) WithTransactionResult[T any](ctx context.Context, fn func(tx *gorm.DB) (T, error)) (T, error) {
//	var result T
//	err := tm.db.WithContext(ctx).Transaction(func(tx *gorm.DB) error {
//		var err error
//		result, err = fn(tx)
//		return err
//	})
//	return result, err
//}

// UnitOfWork 工作单元
type UnitOfWork struct {
	tx *gorm.DB
}

// NewUnitOfWork 创建工作单元
func (tm *Manager) NewUnitOfWork(ctx context.Context) *UnitOfWork {
	return &UnitOfWork{
		tx: tm.db.WithContext(ctx).Begin(),
	}
}

// GetDB 获取事务数据库连接
func (uow *UnitOfWork) GetDB() *gorm.DB {
	return uow.tx
}

// Commit 提交事务
func (uow *UnitOfWork) Commit() error {
	return uow.tx.Commit().Error
}

// Rollback 回滚事务
func (uow *UnitOfWork) Rollback() error {
	return uow.tx.Rollback().Error
}
