package main

import (
	"flag"
	"fmt"

	"git.chillcy.com/golang/colorway/internal/rpc/internal/config"
	colorwayServer "git.chillcy.com/golang/colorway/internal/rpc/internal/server/colorway"
	productServer "git.chillcy.com/golang/colorway/internal/rpc/internal/server/product"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc"
	"git.chillcy.com/golang/colorway/internal/rpc/pb/colorway"

	"github.com/zeromicro/go-zero/core/conf"
	"github.com/zeromicro/go-zero/core/service"
	"github.com/zeromicro/go-zero/zrpc"
	"google.golang.org/grpc"
	"google.golang.org/grpc/reflection"
)

var configFile = flag.String("f", "internal/rpc/etc/dev.yaml", "the config file")

func main() {
	flag.Parse()

	var c config.Config
	conf.MustLoad(*configFile, &c)
	ctx := svc.NewServiceContext(c)

	s := zrpc.MustNewServer(c.RpcServerConf, func(grpcServer *grpc.Server) {
		colorway.RegisterColorwayServer(grpcServer, colorwayServer.NewColorwayServer(ctx))
		colorway.RegisterProductServer(grpcServer, productServer.NewProductServer(ctx))

		if c.Mode == service.DevMode || c.Mode == service.TestMode {
			reflection.Register(grpcServer)
		}
	})
	defer s.Stop()

	fmt.Printf("Starting rpc server at %s...\n", c.ListenOn)
	s.Start()
}
