package cache

import (
	"context"
	model "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/brand_data"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo/options"
	"sync"
	"time"
)

type BrandRank struct {
	brandDataModel model.BrandDataModel
	rwMutex        *sync.RWMutex
	topBrands      []string
	middleBrands   []string
}

func NewBrandRank(brandDataModel model.BrandDataModel) *BrandRank {
	return &BrandRank{brandDataModel: brandDataModel, rwMutex: new(sync.RWMutex)}
}

func (receiver *BrandRank) Refresh(ctx context.Context) error {
	opt := options.Find()
	opt.SetProjection(bson.D{{"_id", 1}})
	opt.SetSort(bson.D{{"platformPayAmount.all.dayRange.90", -1}})
	opt.SetLimit(100)
	brandDataList, _, err := receiver.brandDataModel.Find(ctx, bson.M{}, opt)
	if err != nil {
		return err
	}
	top := brandDataList[:30]
	topIds := make([]string, 0)
	for _, data := range top {
		topIds = append(topIds, data.Id)
	}

	middleIds := make([]string, 0)
	if len(brandDataList) > 30 {
		middle := brandDataList[30:]
		for _, data := range middle {
			middleIds = append(middleIds, data.Id)
		}
	}
	receiver.setBrandRankIds(topIds, middleIds)
	return nil
}

func (receiver *BrandRank) setBrandRankIds(topBrands, middleBrands []string) {
	receiver.rwMutex.Lock()
	defer receiver.rwMutex.Unlock()
	receiver.topBrands = topBrands
	receiver.middleBrands = middleBrands
}

func (receiver *BrandRank) GetBrandRankIds() ([]string, []string) {
	receiver.rwMutex.RLock()
	defer receiver.rwMutex.RUnlock()

	return receiver.topBrands, receiver.middleBrands
}

func (receiver *BrandRank) Interval() time.Duration {
	return time.Hour
}
