package cache

import (
	"context"
	"fmt"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/client/source"
	model "git.chillcy.com/golang/colorway/internal/pkg/database/mongo/model/brand_data"
	"github.com/zeromicro/go-zero/core/logc"
	"go.uber.org/multierr"
	"log"
	"time"
)

var _cache *Cache

type Cache struct {
	refreshMap map[RefreshKey]Refresh
	Source     *Source
}

func InitCache(config Config) {
	s := NewSource(config.Source)
	br := NewBrandRank(config.BrandDataModel)
	// 注册缓存刷新 Map
	_cache = &Cache{
		refreshMap: map[RefreshKey]Refresh{
			RefreshKeySource:       s,
			RefreshKeyRankBrandIds: br,
		},
		Source: s,
	}
	// 初始化第一次加载缓存
	err := _cache.Notify(RefreshKeySource, RefreshKeyRankBrandIds)
	if err != nil {
		panic(err)
	}
	_cache.startRefresh()
}

func (receiver *Cache) Notify(keys ...RefreshKey) error {
	var err error
	for _, key := range keys {
		refresh, ok := receiver.refreshMap[key]
		if !ok {
			err = multierr.Append(err, fmt.Errorf("缺失 %s 对应的更新对象", key))
		}
		e := refresh.Refresh(context.TODO())
		if e != nil {
			err = multierr.Append(err, e)
		}
		logc.Infof(context.TODO(), "缓存更新 %s", key)
	}
	return err
}

func (receiver *Cache) startRefresh() {
	for refreshKey, refresh := range receiver.refreshMap {
		go func(k RefreshKey, r Refresh) {
			for {
				interval := r.Interval()
				time.Sleep(interval)
				err := receiver.Notify(k)
				if err != nil {
					log.Println(err)
					continue
				}
			}

		}(refreshKey, refresh)
	}
}

type RefreshKey string

const (
	RefreshKeySource       = "source"
	RefreshKeyRankBrandIds = "rankBrandIds"
)

type Config struct {
	Source         source.Source
	BrandDataModel model.BrandDataModel
}

func GetCache() *Cache {
	return _cache
}
