package cache

import (
	"context"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/client/source"
	"sync"
	"time"
)

type Source struct {
	items   SourceItems
	rwMutes *sync.RWMutex
	source  source.Source
}

func NewSource(source source.Source) *Source {
	return &Source{source: source, rwMutes: &sync.RWMutex{}}
}

func (receiver *Source) Refresh(ctx context.Context) error {
	receiver.rwMutes.Lock()
	defer receiver.rwMutes.Unlock()

	sources, err := receiver.source.List(ctx, &source.SourceListRequest{})
	if err != nil {
		return err
	}
	var sourceItems SourceItems
	for _, s := range sources.GetSources() {
		sourceItems = append(sourceItems, SourceItem{
			Id: s.GetId(),
			CombineWeight: CombineWeight{
				Image:        int(s.GetCombineWeight().GetImage()),
				Title:        int(s.GetCombineWeight().GetTitle()),
				Category:     int(s.GetCombineWeight().GetCategory()),
				Compositions: int(s.GetCombineWeight().GetComposition()),
				Care:         int(s.GetCombineWeight().GetCare()),
				Description:  int(s.GetCombineWeight().GetDescription()),
				Fits:         int(s.GetCombineWeight().GetFits()),
				Models:       int(s.GetCombineWeight().GetModels()),
				Season:       int(s.GetCombineWeight().GetSeason()),
			},
		})
	}
	receiver.items = sourceItems
	return nil
}

func (receiver *Source) Interval() time.Duration {
	return time.Hour
}

func (receiver *Source) GetSourceItems() SourceItems {
	receiver.rwMutes.RLock()
	defer receiver.rwMutes.RUnlock()

	return receiver.items
}

type SourceItem struct {
	Id            string
	CombineWeight CombineWeight
}

type SourceItems []SourceItem

type CombineWeight struct { // https://trello.com/c/vrEKHL8e
	Image        int `json:"image" bson:"image" form:"image"`
	Title        int `json:"title" bson:"title" form:"title"`
	Category     int `json:"category" bson:"category" form:"category"`
	Compositions int `json:"compositions" bson:"compositions" form:"compositions"`
	Care         int `json:"care" bson:"care" form:"care"`
	Description  int `json:"description" bson:"description" form:"weight_desc"`
	Fits         int `json:"fits" bson:"fits" form:"fits"`
	Models       int `json:"models" bson:"models" form:"models"`
	Season       int `json:"season" bson:"season" form:"season"`
}
