package svc

import (
	"git.chillcy.com/golang/chillcy/pkg/etcd"
	rate2 "git.chillcy.com/golang/chillcy/pkg/sdk/rate"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/client/product"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/client/source"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mongo"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/config"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/cache"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/convert"
	"git.chillcy.com/golang/colorway/internal/rpc/internal/svc/rate"
	"github.com/zeromicro/go-zero/zrpc"
)

type ServiceContext struct {
	Config         config.Config
	MongoManager   *mongo.Manager
	Pc4RpcService  Pc4RpcService
	Cache          *cache.Cache
	SpuConvert     *convert.Spu
	ProductConvert *convert.Product
}

func NewServiceContext(c config.Config) *ServiceContext {
	etcd.Init(etcd.Env(c.Mode))
	pc4RpcProductServiceTarget := etcd.Get(etcd.KeyRpcPc4)
	mongoManager := mongo.NewManager(etcd.Get(etcd.KeyMongoProductUri))
	err := mongoManager.StartCache()
	if err != nil {
		panic(err)
	}
	if etcd.GetEnv() == etcd.EnvDev {
		etcd.SetEtcd(etcd.KeyHttpRateApiUri.String(), "http://127.0.0.1:6060")
	}
	svc := &ServiceContext{
		Config:       c,
		MongoManager: mongoManager,
		Pc4RpcService: Pc4RpcService{
			Product: product.NewProduct(zrpc.MustNewClient(zrpc.RpcClientConf{Target: pc4RpcProductServiceTarget})),
			Source:  source.NewSource(zrpc.MustNewClient(zrpc.RpcClientConf{Target: pc4RpcProductServiceTarget})),
		},
	}
	cache.InitCache(cache.Config{
		Source:         svc.Pc4RpcService.Source,
		BrandDataModel: mongoManager.BrandDataModel,
	})
	svc.Cache = cache.GetCache()
	svc.SpuConvert = convert.NewSpu(svc.Cache)
	rateClient := rate2.NewClient(etcd.Get(etcd.KeyHttpRateApiUri))
	brandRank := cache.NewBrandRank(svc.MongoManager.BrandDataModel)
	svc.ProductConvert = convert.NewProduct(svc.Cache, svc.MongoManager, rate.NewRate(rateClient), brandRank)
	return svc
}

type Pc4RpcService struct {
	Product product.Product
	Source  source.Source
}
