package main

import (
	"context"
	"encoding/json"
	"flag"
	"git.chillcy.com/golang/chillcy/pkg/etcd"
	"git.chillcy.com/golang/chillcy/pkg/rabbitmq"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/message"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mongo"
	"git.chillcy.com/golang/colorway/internal/rpc/client/product"
	"github.com/streadway/amqp"
	"github.com/zeromicro/go-zero/zrpc"
	"log"
)

var _env = flag.String("env", "dev", "环境配置")
var _databaseManager *mongo.Manager
var _consumer *rabbitmq.Consumer
var _productService product.Product

func init() {
	flag.Parse()
	etcd.Init(etcd.Env(*_env))
	_databaseManager = mongo.NewManager(etcd.Get(etcd.KeyMongoProductUri))
	_consumer = rabbitmq.NewRabbitmqConsumer(rabbitmq.Option{
		RabbitMqUri:      etcd.Get(etcd.KeyRabbitMQClusterNode01Uri),
		QueueName:        "product.info.events.sync",
		ConsumerTag:      "product",
		ExchangeName:     message.ProductInfoExchange,
		ExchangeType:     amqp.ExchangeDirect,
		BindingKey:       "#",
		ConsumerWorker:   5,
		ConsumerPrefetch: 5,
	})
	_productService = product.NewProduct(zrpc.MustNewClient(zrpc.RpcClientConf{
		Target: "127.0.0.1:5200",
	}))
}

func main() {
	log.Println("脚本开始")
	defer log.Println("脚本结束")
	err := _consumer.HandlerMessage(func(msg []byte) error {
		var productInfoMsg message.ProductInfo
		err := json.Unmarshal(msg, &productInfoMsg)
		if err != nil {
			return err
		}
		if productInfoMsg.Operation == message.ProductInfoOperationInsert {
			_, err = _productService.Upload(context.TODO(), &product.ProductUploadRequest{
				StyleNumbers: []string{productInfoMsg.DesignerId},
			})
			if err != nil {
				log.Println(string(msg))
				return err
			}
		}
		return nil
	})
	if err != nil {
		panic(err)
	}
}
