package main

import (
	"context"
	"flag"
	"git.chillcy.com/golang/chillcy/pkg/etcd"
	"git.chillcy.com/golang/chillcy/project/pc4/pkg/rpc/client/category"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mysql"
	"git.chillcy.com/golang/colorway/internal/pkg/database/mysql/model"
	"github.com/zeromicro/go-zero/zrpc"
	"log"
)

var _env = flag.String("env", "dev", "环境配置")
var _categoryService category.Category
var _databaseManager *mysql.Manager

func init() {
	flag.Parse()
	etcd.Init(etcd.Env(*_env))
	pc4RpcProductServiceTarget := etcd.Get(etcd.KeyRpcPc4)
	if etcd.GetEnv() == etcd.EnvDev {
		pc4RpcProductServiceTarget = "127.0.0.1:5100"
	}
	_categoryService = category.NewCategory(zrpc.MustNewClient(zrpc.RpcClientConf{Target: pc4RpcProductServiceTarget}))
	// todo dsn etcd
	dsn := "root:root@tcp(127.0.0.1:3306)/colorway?charset=utf8mb4&collation=utf8mb4_unicode_ci&parseTime=true&loc=Local"
	var err error
	_databaseManager, err = mysql.NewManager(dsn)
	if err != nil {
		panic(err)
	}
}

func main() {
	log.Println("脚本开始")
	defer log.Println("脚本结束")

	categoryList, err := _categoryService.List(context.TODO(), &category.CategoryListRequest{})
	if err != nil {
		panic(err)
	}

	var categories model.Categories

	for _, item := range categoryList.GetItems() {
		categories = append(categories, model.Category{
			Code:       item.GetId(),
			Name:       item.GetLabel(),
			ParentCode: item.GetParentId(),
		})
	}

	err = _databaseManager.CategoryRepository.UpsertManyByCode(categories)
	if err != nil {
		panic(err)
	}
}
